      SUBROUTINE FLXODF(KSC,KA1,KA2,OPAKEY,T,WL,L3,FNUE,BLK,
     A                  LMX,KMAX)

C*************************************************************
C
C      CALCULATES MONOCROMATIC AND INTEGRATED FLUX AND SOME 
C      INTEGRALES FOR TEMPERATURE CORRECTION
C      INCLUDES OPACITY DISTRIBUTION FUNCTIONS
C
C*************************************************************

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ALPHA(7,50),WL(LMX),FREQ(500),FNUE(LMX),BLK(LMX),
     1 T(50),ZET(50),
     1 TAU(50),TA(50),PP(50),PPE(50),CAP4(50),CAC(50),CAP(50),          
     2   CAPP(50),CAPJ(50),CAPF(50),SIG(50),SOUR(50),   
     3   TENS(50),FL0(50),BN(50),FLX(50),TNS(50),GRAD(50),              
     4   FR(15,50),X(4,4),ALQ(50),DELB1(50),DELB2(50),B(50),            
     5   BKT(50),EN(50),FL0C(50),TENSC(50),FL0L(50),RESIDI(50),
     6   TNS0(50),CAP0(50),SIGEL(50)
      DIMENSION SIGM(50)                                                     
      CHARACTER*1 CONKEY
      CHARACTER*80 OPAKEY
C
C***  KEY OPAKEY IS USED TO SWITCH BETWEEN SOLELY CONTINUOUS AND
C***             CONTINUOUS PLUS ODF-OPACITIES
C***  KEY CONKEY IS USED TO DETECT LOW OPACITIES IN ODF-TABLES AND 
C***             DROP EXPLICIT RADIATIVE TRANSFER IN THESE CASES TO 
C***             COMPUTING TIME.
C
      COMMON /MODEL/    TEF,G,ALPHA,WM                                      
      COMMON /PARAM/    LA,LL,NN,KK                                        
      COMMON /OPTDEP/   ZET,TAU,P,Q,ALQ                                   
      COMMON /STN_TAUN/ TA,Z                                              
      COMMON /DRUCK/    PP,PPE,FR                                          
      COMMON /ABSKO/    CAP4,SIG                                           
      COMMON /INTGRA/   FLX,GRAD,TNS,CAPP,CAPJ,CAPF,DELB1,DELB2,B
      COMMON /ODF/      CAPODF(50)

C  Local constants.
      DATA C  / 2.9979 D+10 /
      DATA PI / 3.1415 92653 58979 D+00 /
      DATA SIGMA / 5.6692 D-05 /
      DATA BK    / 1.38054 D-16 /

C.

      KMAX  = 0
      DO I = LA,LL
         FREQ(I) = C / WL(I) * 1.D+8
      ENDDO
      DO K = 1,NN
         SIGM(K) = SIG(K)
         FLX(K)  = 0.
         GRAD(K) = 0.
         TNS(K)  = 0.
         B(K)    = 0.
         CAPP(K) = 0.
         CAPJ(K) = 0.
         CAPF(K) = 0.
         BKT(K)  = BK * T(K)
         EN(K)   = PPE(K) / BKT(K)
C     Thomson scattering.
         SIGEL(K) = 0.6665E-24*PPE(K)/((PP(K)-PPE(K))*WM)
      ENDDO
      NSTEPS = 10
C  Starting approximation: S = B.
      KSC=1


C******************************************************************
C      LOOP OVER FREQUENCIES
C******************************************************************
C
      DO I = LA,LL
*       IF (KSC.EQ.2) READ(KA1) (SOUR(K), K = 1,NN)

C     Opacities.
        DO K=1,NN
          CAPODF(K)=0.0
          SIG(K) = 0.0
        ENDDO
        CALL KONTOP(WL(I),WL,L3,T,X,I,ISW,CAC)
        DO K=1,NN
          SIG(K) = SIG(K) + SIGEL(K)
          CAP(K) = CAC(K)
        ENDDO

C    Continuum source function.
        CALL TAUNUE ( TAU, ALQ, CAP, SIG, CAP4, NN, TA )
        CALL AVLO(KSC,BN,T,FREQ(I),CAC,SIG,SOUR,FL0C,KI,TENSC,
     :            CAP4,KJ)
*       WRITE(KA2) (SOUR(K), K = 1,NN)                                  
        W = FREWGT(I,FREQ)

C    Continum only.
        IF (OPAKEY.EQ.'CONTINUUM') THEN
          DO K=1,NN
            FL0(K)  = FL0C(K)
            TENS(K) = TENSC(K)
            FLX(K)  = FLX(K) +W*FL0(K)
            B(K)    = B(K)   +W*BN(K)
            CAPP(K) = CAPP(K)+W*CAP(K)*BN(K)
            CAPJ(K) = CAPJ(K)+W*CAP(K)*TENS(K)
            CAPF(K) = CAPF(K)+W*(CAP(K)+SIG(K))*FL0(K)
            TNS(K)  = TNS(K) +W*TENS(K)
          ENDDO 

*          write (6,'(f7.0,1p10e9.2,9(/a7,10e9.2))')
*     :          wl(i),(ta(j),j=1,5),(cac(j),j=10,50,10),
*     :         '  cap  ',(cac(j),j=1,5),(cac(j),j=10,50,10),
*     :         '  sig  ',(sig(j),j=1,5),(sig(j),j=10,50,10),
*     :         '  bn   ',(bn(j),j=1,5),(bn(j),j=10,50,10),
*     :         '  sour ',(sour(j),j=1,5),(sour(j),j=10,50,10),
*     :         '  fl0c ',(fl0c(j),j=1,5),(fl0c(j),j=10,50,10),
*     :         '  tens ',(tens(j),j=1,5),(tens(j),j=10,50,10),
*     :         '  tns  ',(tns(j),j=1,5),(tns(j),j=10,50,10)
*

C     Include line opacities from odfs (ordered from low-to-high opacity).
        ELSE
          DO K=1,NN
            FL0(K) = 0.0
            TNS0(K) = 0.0
            CAP0(K) = 0.0
          ENDDO
          SUMWT = 0.0
          CONKEY = 'N'
          DO N=1,NSTEPS
            CALL LINOP(T,EN,FREQ(I),N,NSTEPS,STEPWT)
            IF (CONKEY.EQ.'N') THEN
              DO K=1,NN
                CAP(K) = CAC(K)+CAPODF(K)
              ENDDO
              CALL TAUNUE ( TAU, ALQ, CAP, SIG, CAP4, NN, TA )
C           START ITERATION WITH CONTINUUM SOURCE FUNCTION
              CALL AVLO(KSC,BN,T,FREQ(I),CAP,SIG,SOUR,FL0L,KI,
     :                  TENS,CAP4,KJ)   
            ELSE
              DO K=1,NN
                FL0L(K) = FL0C(K)
                TENS(K) = TENSC(K)
              ENDDO
            ENDIF

C         Frequency integration
            DO K=1,NN
              RESIDI(K) = FL0L(K) / FL0C(K)
              FL0(K) = FL0(K) + FL0L(K)*STEPWT
              FLX(K) = FLX(K)+W*FL0L(K)*STEPWT
              B(K) = B(K)+W*BN(K)*STEPWT
              CAPP(K) = CAPP(K)+W*CAP(K)*BN(K)*STEPWT
              CAPJ(K) = CAPJ(K)+W*CAP(K)*TENS(K)*STEPWT
              CAPF(K) = CAPF(K)+W*(CAP(K)+SIG(K))*FL0L(K)*STEPWT
              TNS(K) = TNS(K)+W*TENS(K)*STEPWT
            ENDDO
            SUMWT = SUMWT + STEPWT
          ENDDO
          RES = FL0(1)/FL0C(1)
          BLK(I) = 1-RES
        ENDIF

        FNUE(I) = FL0(1)
        IF (KJ .GT. KMAX) KMAX = KJ
      ENDDO

C*************************************************************
C       END OF FREQUENCY LOOP
C*************************************************************

      DO K = 1,NN                                           
         GRAD(K)= PI/C/CAP4(K)*CAPF(K)                                   
         SIG(K) = SIGM(K)                                               
      ENDDO

      END                             
