        SUBROUTINE ACCEL(Y,N,YACC)
C**** SECOND ORDER ACCELERATION OF CONVERGENCE
C**** (L. AUER, 1987, IN NUMERICAL RADIATIVE TRANSFER,
C**** ED. W. KALKOFEN, P.101)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION Y(0:3,N),YACC(N)
        A1 = 0.
        B1 = 0.
        B2 = 0.
        C1 = 0.
        C2 = 0.
        DO 100 I=1,N
           WT = 1./Y(0,I)
           D0 = Y(0,I)-Y(1,I)
           D1 = Y(0,I)-2.*Y(1,I)+Y(2,I)
           D2 = Y(0,I)-Y(1,I)-Y(2,I)+Y(3,I)
           A1 = A1+WT*D1*D1
           B1 = B1+WT*D1*D2
           C1 = C1+WT*D0*D1
           B2 = B2+WT*D2*D2
           C2 = C2+WT*D0*D2
100     CONTINUE
C****   COMPUTE COEFFICIENTS
        A = (B2*C1-B1*C2) / (B2*A1-B1*B1)
        B = (A1*C2-B1*C1) / (B2*A1-B1*B1)
C****   ACCELERATE
        DO 200 I=1,N
           YACC(I) = (1.-A-B)*Y(0,I) + A*Y(1,I) + B*Y(2,I)
200     CONTINUE
        RETURN
        END

      SUBROUTINE ALKOEF(S,N,C)                                          
C***  BERECHNUNG DER AVRETT-LOESER-KOEFFIZIENTEN                        
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION FAV(50,50)
      DIMENSION S(N),C(N)                                               
      DIMENSION TA(50)                                                  
      COMMON /STN_TAUN/ TA,Z                                              
      NM1 = N-1                                                         
      NM2 = N-2                                                         
C***  BERECHNUNG DER AVRETT-LOESER-PARABELN                             
      DO 100 J=1,N
         FAV(J,1) = 1.
         FAV(J,N) = TA(J)
         DO 110 I=2,NM1
            IF ((TA(I)-TA(J)).GT.0.) THEN
               FAV(J,I) = (1.-TA(J)/TA(I))*(1.-Z*TA(J)/TA(I))
            ELSE
               FAV(J,I) = 0.
            ENDIF
110      CONTINUE
100   CONTINUE
      C(N) = (S(N)-S(NM1))/(FAV(N,N)-FAV(NM1,N))                        
      C(1) = S(NM1)-C(N)*FAV(NM1,N)                                     
      DO 20 J = 1,NM2                                                   
         I = N-J                                                        
         IM1 = I-1                                                      
         IP1 = I+1                                                      
         SUM = 0.                                                       
         DO 10 K = IP1,N                                                
   10       SUM = SUM+C(K)*FAV(IM1,K)                                   
   20    C(I) = (S(IM1)-C(1)-SUM)/FAV(IM1,I)                            
      RETURN                                                            
      END                                                               

      SUBROUTINE AVLO(KSC,BN,T,FRE,CA,SIG,SOUR,FL0,KI,EPS,TENS,CAP4,KJ) 
C**** ITERATIVE BERECHNUNG DER SOURCE FUNCTION UND DES                  
C**** MONOCHROMATISCHEN FLUSSES NACH DER METHODE VON AVRETT UND         
C**** LOESER JQSRT 201, 3 (1963)                                        
C**** NULLTE NAEHERUNG WAHLWEISE SNUE = BNUE, KSC = 1                   
C**** ODER DIE NULLTE NAEHERUNG WIRD VON DER VORHERGEHENDEN             
C**** TEMPERATURSCHICHTUNG UEBERNOMMEN, KSC = 2. SOUR IST AUF           
C**** TAPE 2 ODER TAPE 3 (ABWECHSELND) GESPEICHERT                      
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION BN(50),T(50),CA(50),SIG(50),SOUR(50),TENS(50),TA(50),   
     1   RHO(50),SORC(50),C(50),FL0(50),HH(50,50),GG(50,50),CAP4(50)    
      COMMON /PARAM/ LA,LL,NN,KK                                        
      COMMON /STN_TAUN/ TA,Z                                              
      COMMON GG,HH                                                      
      COMMON/QSTATIS/ KIMAX,KST,FRST(5),ERST(5)
      DATA Z/0.9/                                                       
      ERR0 = 1000000.
      DO 100 I = 1,NN                                                   
  100    RHO(I) = SIG(I)/(SIG(I)+CA(I))                                 
      DO 101 J = 1,NN                                                   
         KJ = J                                                         
         IF (TA(J) .GT. 7.0)  GOTO 102
  101    CONTINUE                                                       
  102 CALL BNUE(T,FRE,BN,NN)                                            
      KJ=MAX0(KJ,10)
      GOTO (110,130) KSC                                                
  110 CONTINUE                                                          
  111 DO 120 I = 1,NN                                                   
  120    SOUR(I) = BN(I)                                                
  130 CALL GH(GG,HH,KJ)                                                 
C***  BEGINN DER ITERATION                                              
      KI = 1                                                            
  140 CALL ALKOEF(SOUR,KJ,C)                                            
      DO 150 J = 1,KJ                                                   
         TENS(J) = 0.                                                   
         DO 150 I = 1,KJ                                                
  150    TENS(J) = TENS(J)+GG(J,I)*C(I)                                 
         KS = 1                                                         
      DO 152 J = 1,KJ                                                   
            IF (TENS(J) .GE. 0.) GOTO 151                               
            IF (J .EQ. 1) L = J                                         
            KS = 0                                                      
            K = L                                                       
            GOTO 152                                                    
  151       L = J                                                       
            IF (KS .EQ. 0) CALL INTP(K,L,TENS,TA,KJ,KS)                 
  152       CONTINUE                                                    
      DO 160 J = 1,KJ                                                   
  160    SORC(J) = (1.-RHO(J))*BN(J)+RHO(J)*TENS(J)                     
      CALL FEHLER(SOUR,SORC,KJ,KJ,ERR)                                  
      IF (ERR .LT. ERR0)  GOTO 200                                      
        GOTO 242                                                        
  200 DO 210 I = 1,KJ                                                   
  210    SOUR(I) = SORC(I)                                              
C       IF (FRE.GT.3.07E15.AND.FRE.LT.3.08E15)
C     A      WRITE(6,*) 'KI= ',KI,'ERR = ',ERR
      IF (ERR .LE.  0.0001 )  GOTO 240
      KI = KI+1                                                         
      ERR0 = ERR                                                        
      IF (KI.GE.KIMAX) GOTO 242
      GOTO 140                                                          
 240  IF (ERR .LE. 0.0001 .OR. Z .EQ. 0.9)  GOTO 241
      Z = 0.9                                                           
      GOTO 111                                                          
242   KST = KST+1
      DO 243 I=1,5
243   IF (ERR.GT.ERST(I)) GOTO 244
      GOTO 241
244   IF (I.EQ.5) GOTO 246
      DO 247 J=I,4
      ERST(I+1) = ERST(I)
247   FRST(I+1) = FRST(I)
246   ERST(I) = ERR
      FRST(I) = FRE
C***  FLUSSBERECHNUNG                                                   
  241 CALL ALKOEF(SOUR,KJ,C)                                            
      DO 250 J = 1,KJ                                                   
         FL0(J) = 0.                                                    
         DO 250 I = 1,KJ                                                
  250    FL0(J) = FL0(J)+HH(J,I)*C(I)                                   
C***  DIFFUSIONSNAEHERUNG FUER TA .GT. 5.                               
      IF (KJ.NE.NN) THEN
        NM1 = NN-1                                                        
        KJP1 = KJ+1                                                       
        DO  J = KJ,NN                                                  
          TENS(J) = BN(J)                                                
          SOUR(J) = BN(J)                                                
        ENDDO
        DO  J = KJ,NM1                                                 
          JM1 = J-1                                                      
          JP1 = J+1                                                      
          FL0(J) = 0.5*((BN(J)-BN(JM1))/(TA(J)-TA(JM1))+(BN(JP1)-BN(J))/ 
     1             (TA(JP1)-TA(J)))*4./3.                                   
        ENDDO
        FL0(NN) = (BN(NN)-BN(NM1))/(TA(NN)-TA(NM1))*4./3.                 
      ENDIF
*     write (6,1200) FL0
1200  format (1P10e11.2)
      KS = 1                                                            
      DO J = 1,NN                                                   
         IF (FL0(J) .LT. 0.) THEN
           IF (J .EQ. 1) L = J                                            
           KS = 0                                                         
           K = L                                                          
         ELSE
           L = J                                                          
           IF (KS .EQ. 0) CALL INTP(K,L,FL0,TA,NN,KS)                     
         ENDIF
      ENDDO
*     write (6,1200) FL0
      RETURN                                                            
      END                                                               


      SUBROUTINE BAND(WL,LA,LL,WL5,L3,LMAX)
C**** FESTLEGUNG DES WLLENLAENGENBANDES , IN DEM DAS SYNTHETISCHE SPEK-
C**** TRUMS BERECHNET WIRD
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION WL(LMAX),WL5(L3)
      COMMON /SYNTHE/ WLLA,WLLL,IFI
      COMMON/PARKON/ LA3
      IF (IFI.EQ.0) GOTO 100
      DO 10 J=LA,L3
         IF (WL5(J).GT.WLLA) GOTO 20
10    CONTINUE
20    IF (J.EQ.LA) J=LA+1
      LA3 = J-1
      WLLA = WL5(J-1)
      DO 30 I= LA,LL
         IF (ABS(WL(I)-WLLA).LT.0.01) GOTO 40
30    CONTINUE
40    LA=I
      DO 50 J=LA3,L3
         IF (WL5(J).GT.WLLL) GOTO 60
         IF (J.EQ.L3) GOTO 60
50    CONTINUE
60    LL3 = J
      WLLL = WL5(J)
      DO 70 I= LA,LL
         IF (ABS(WL(I)-WLLL).LT.0.01) GOTO 80
70    CONTINUE
80    LL=I
      WRITE(6,1010)
      RETURN
100   LA3=LA
      RETURN
1010  FORMAT(/,3X,'ACHTUNG,NUR SYNTHETISCHES SPEKTRUM WIRD BERECHNET(KEI
     1N LINE-BLANKETING')
      END

      SUBROUTINE BNUE(T,FRE,BN,NN)                                      
C**** PLANCKFUNKTION (NACH BOEHM UND DEINZER (Z.ASTROPHYS.61,1,(1965))) 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION T(NN),BN(NN)                                            
      HH = 4.79930E-11                                                  
      H2 = 1.32504E-26                                                  
      CC2 = 8.98758E+20                                                 
      CBC = (H2*FRE**2)*FRE/CC2                                         
      DO 35 I = 1,NN                                                    
      EP = HH*FRE/T(I)                                                  
   33 BN(I) = CBC/(EXP(EP)-1.)                                          
   35 CONTINUE                                                          
      RETURN                                                            
      END                                                               

      SUBROUTINE FEHLER(S1,S2,NN,MM,ERR)                                
C***  BERECHNUNG DES RELATIVEN FEHLERS EINER FUNKTION S                 
C***  DIE BEIDEN NAEHERUNGEN STEHEN AUF S1 (DIM. = NN) UND S2 (DIM. =   
C***  NN ODER 1, D. H. S2 IST EINE KONSTANTE)                           
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION S1(NN),S2(MM)                                           
      DELT = 0.                                                         
      DO 20 J = 1,NN                                                    
      IF (MM .EQ. NN) DEL = ABS((S2(J)-S1(J))/S2(J))                    
      IF (MM .EQ. 1 ) DEL = ABS((S2(1)-S1(J))/S2(1))                    
      IF (DELT .LT. DEL) DELT = DEL                                     
20    CONTINUE                                                          
      ERR = DELT                                                        
      RETURN                                                            
      END                                                               

      SUBROUTINE FLXODF(KSC,KA1,KA2,OPAKEY,EPS,T,WL,WL5,L3,FNUE,BLK,
     A                  LMX,KMAX)
C
C*************************************************************
C
C      CALCULATES MONOCROMATIC AND INTEGRATED FLUX AND SOME 
C      INTEGRALES FOR TEMPERATURE CORRECTION
C      INCLUDES OPACITY DISTRIBUTION FUNCTIONS
C
C*************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ALPHA(7,50),WL(LMX),FREQ(500),FNUE(LMX),BLK(LMX),
     1 T(50),ZET(50),
     1 TAU(50),TA(50),PP(50),PPE(50),CAP4(50),CAC(50),CAP(50),          
     2   CAPP(50),CAPJ(50),CAPF(50),SIG(50),SOUR(50),   
     3   TENS(50),FL0(50),BN(50),FLX(50),TNS(50),GRAD(50),              
     4   FR(15,50),X(4,4),ALQ(50),DELB1(50),DELB2(50),B(50),            
     5   BKT(50),EN(50),WL5(L3),FL0C(50),TENSC(50),FL0L(50),RESIDI(50),
     6   TNS0(50),CAP0(50),SIGEL(50)
      DIMENSION SIGM(50)                                                     
      CHARACTER*1 CONKEY
      CHARACTER*80 OPAKEY
C
C***  KEY OPAKEY IS USED TO SWITCH BETWEEN SOLELY CONTINUOUS AND
C***             CONTINUOUS PLUS ODF-OPACITIES
C***  KEY CONKEY IS USED TO DETECT LOW OPACITIES IN ODF-TABLES AND 
C***             DROP EXPLICIT RADIATIVE TRANSFER IN THESE CASES TO 
C***             COMPUTING TIME.
C
      COMMON /MODEL/    TEF,G,ALPHA,WM                                      
      COMMON /PARAM/    LA,LL,NN,KK                                        
      COMMON /OPTDEP/   ZET,TAU,P,Q,ALQ                                   
      COMMON /STN_TAUN/ TA,Z                                              
      COMMON /DRUCK/    PP,PPE,FR                                          
      COMMON /ABSKO/    CAP4,SIG                                           
      COMMON /INTGRA/   FLX,GRAD,TNS,CAPP,CAPJ,CAPF,DELB1,DELB2,B
      COMMON /ODF/      CAPODF(50)

      C     = 2.99793E+10
      PI    = 4.*ATAN(1.)                                                  
      SIGMA = 5.6692E-05                                                
      KMAX  = 0
      BK    = 1.38054E-16                                                    
      DO 10 I = LA,LL
         FREQ(I) = C / WL(I) * 1.E+8
10    CONTINUE
      DO 100 I = 1,NN                                                   
         SIGM(I) = SIG(I)                                               
         FLX(I)  = 0.                                                    
         GRAD(I) = 0.                                                   
         TNS(I)  = 0.                                                    
         B(I)    = 0.                                                      
         CAPP(I) = 0.                                                   
         CAPJ(I) = 0.                                                   
         CAPF(I) = 0.                                                   
100   CONTINUE
      DO 110 K=1,NN
         BKT(K) = BK*T(K)
         EN(K)  = PPE(K)/BKT(K)
110   CONTINUE
      NSTEPS = 10
C***  TEST MIT S=B ALS STARTNAEHERUNG
      KSC=1
C***  THOMSON-SCATTERING
      DO 12 K = 1,NN
         SIGEL(K) = 0.6665E-24*PPE(K)/((PP(K)-PPE(K))*WM)
12    CONTINUE
C
C******************************************************************
C      LOOP OVER FREQUENCIES
C******************************************************************
C
      DO 120 I = LA,LL                                                  
        IF (KSC.EQ.2) READ(KA1) (SOUR(K), K = 1,NN)
        DO 130 K=1,NN                                           
          CAPODF(K)=0.0                                                        
130     CONTINUE
        DO 140 K=1,NN                                                   
          SIG(K) = 0.0                                                 
140     CONTINUE
        CALL KONTOP(WL(I),WL5,L3,T,X,I,ISW,CAC)                         
*       write (6,*) CAC
        DO 150 K=1,NN                                                   
          SIG(K) = SIG(K) + SIGEL(K)
          CAP(K) = CAC(K)
150     CONTINUE

C*** Calculate continuum source function.
        CALL TAUNUE(CAP)
        CALL AVLO(KSC,BN,T,FREQ(I),CAC,SIG,SOUR,FL0C,KI,EPS,TENSC,
     A            CAP4,KJ)      
        WRITE(KA2) (SOUR(J), J = 1,NN)                                  
*       write (6,1200) (T(J),CAC(J),SIG(J),SOUR(J),FL0C(J),J=1,NN)
1200    format (0PF8.1,1P4E12.2)
        W = FREWGT(I,FREQ)

C*** Continum only.
        IF (OPAKEY.EQ.'CONTINUUM') THEN
          DO 152 K=1,NN
            FL0(K) = FL0C(K)
            TENS(K) = TENSC(K)
152       CONTINUE
          HNUE1=FL0(1)/4.
*         write(6,1100) WL(I),HNUE1
          DO 170 J = 1,NN                                              
            FLX(J) = FLX(J)+W*FL0(J)                                 
            B(J) = B(J)+W*BN(J)                                              
            CAPP(J) = CAPP(J)+W*CAP(J)*BN(J)                         
            CAPJ(J) = CAPJ(J)+W*CAP(J)*TENS(J)                               
            CAPF(J) = CAPF(J)+W*(CAP(J)+SIG(J))*FL0(J)
            TNS(J) = TNS(J)+W*TENS(J)                                        
170       CONTINUE 

C*** Including line opacities from odfs (ordered from low-to-high opacity).
        ELSE
          DO 151 K=1,NN
            FL0(K) = 0.0
            TNS0(K) = 0.0
            CAP0(K) = 0.0
151       CONTINUE
          SUMWT = 0.0
          CONKEY = 'N'
          DO 155 N=1,NSTEPS
*           write (6,*) 'FLXODF - LINOP', FREQ(I), N
            CALL LINOP(T,EN,FREQ(I),N,NSTEPS,STEPWT)
*           write (6,*) CAPODF
            IF (CONKEY.EQ.'N') THEN
              DO 160 K=1,NN
                CAP(K) = CAC(K)+CAPODF(K)
160           CONTINUE
              CALL TAUNUE(CAP)
C*****        START ITERATION WITH CONTINUUM SOURCE FUNCTION
              CALL AVLO(KSC,BN,T,FREQ(I),CAP,SIG,SOUR,FL0L,KI,EPS,
     A                  TENS,CAP4,KJ)   
            ELSE
              DO 1553 K=1,NN
                FL0L(K) = FL0C(K)
                TENS(K) = TENSC(K)
1553          CONTINUE
            ENDIF
            DO 156 IN = 1,NN
               RESIDI(IN) = FL0L(IN) / FL0C(IN)
               FL0(IN) = FL0(IN) + FL0L(IN)*STEPWT
156         CONTINUE
C*****      FREQUENCY INTEGRATION
            DO 157 J=1,NN
              FLX(J) = FLX(J)+W*FL0L(J)*STEPWT
              B(J) = B(J)+W*BN(J)*STEPWT
              CAPP(J) = CAPP(J)+W*CAP(J)*BN(J)*STEPWT
              CAPJ(J) = CAPJ(J)+W*CAP(J)*TENS(J)*STEPWT
              CAPF(J) = CAPF(J)+W*(CAP(J)+SIG(J))*FL0L(J)*STEPWT
              TNS(J) = TNS(J)+W*TENS(J)*STEPWT
157         CONTINUE
            HNU1 = FL0L(1)/4.
*           write(6,1000) WL(I),N,HNU1,RESIDI(1)
            SUMWT = SUMWT + STEPWT
155         CONTINUE
            RES = FL0(1)/FL0C(1)
            BLK(I) = 1-RES
            HNU1M=FL0(1)/4.
*           write(6,1100) WL(I),HNU1M,RES
         ENDIF

         FNUE(I) = FL0(1)                                                       
         IF (KJ .GT. KMAX) KMAX = KJ
120   CONTINUE                                                          
C
C*************************************************************
C       END OF FREQUENCY LOOP
C*************************************************************
C
      DO 350 J = 1,NN                                           
         GRAD(J)=PI/C/CAP4(J)*CAPF(J)                                   
         SIG(J) = SIGM(J)                                               
350   CONTINUE
      RETURN                                                            
1000    FORMAT(1X,F10.1,I3,2G12.3)
1100    FORMAT(1X,' WLI, HNUE(1) RES ',1X,F10.1,2G12.3)
      END                             
                                  
        DOUBLE PRECISION FUNCTION FREWGT(I,FREQ)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION FREQ(500)
        COMMON /PARAM/LA,LL,NN,KK
C
C*****  COMPUTE WEIGHTS FOR TRAPEZOIDAL INTEGRATION OVER
C*****  FREQUENCIES
C*****  CALLED FROM FLXODF
C**
        IF (I.EQ.LA) THEN
            W = 0.5*(FREQ(LA)-FREQ(LA+1))
            ELSE
            IF (I.EQ.LL) THEN  
               W = 0.5*(FREQ(LL-1)+FREQ(LL))
            ELSE
               W = 0.5*(FREQ(I-1)-FREQ(I+1))
            ENDIF
        ENDIF
        FREWGT = W
        RETURN
      END

      SUBROUTINE GH(G,H,NN)                                             
C***  ROUTINE IS MERGED FROM ORIGINAL GH AND EXPI IN ORDER TO SAVE 
C***  COMPUTING TIME
C***  BERECHNUNG DER MATRIXELEMENTE DER MITTLEREN INTENSITAET UND       
C***  DES MONOCHROMATISCHEN STRAHLUNGSFLUSSES NACH AVRETT UND LOESER    
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION TA(50),G(50,50),H(50,50),E2(50,50),E3(50,50)
      DIMENSION E4(50,50),E5(50,50),E2J(50),E3J(50),E4J(50),E5J(50)
      COMMON /STN_TAUN/ TA,Z                                              
      NM1 = NN-1                                                        
C **** CALCULATE EXPONENTIAL-INTEGRALS
C
C***** EXPONENTIAL INTEGRALS FOR ALL TA(J)
      DO 1 J=1,NN
          X = TA(J)
          IF (X.GT.50.) THEN
             E2J(J) = 0.0
             E3J(J) = 0.0
             E4J(J) = 0.0
             E5J(J) = 0.0
          ELSE
            EXN = EXP(-X)                                                       
            IF (X .GE. 1.0) THEN
              IF (X .GT. 5.) THEN
                X2 = (X+9.)*(X+9.)                                              
                E9 = EXN/(X+9.)*((((54.*X*X-648.*X+729.)/X2+                    
     1               (81.-18.*X))/X2+9.)/X2+1.)                         
                E8 = (EXN - 8.*E9)/X                                            
                E7 = (EXN - 7.*E8)/X                                            
                E6 = (EXN - 6.*E7)/X                                            
                E5J(J) = (EXN - 5.*E6)/X
                E4J(J) = (EXN - 4.*E5J(J))/X            
                E3J(J) = (EXN - 3.*E4J(J))/X    
                E2J(J) = (EXN - 2.*E3J(J))/X    
              ELSE
                E1 = EXN/X*
     1              (((X + 5.12669020)*X + 4.53079215)*X + 0.2372905)/
     1              (((X + 6.12652717)*X + 8.66601262)*X + 2.47663307)  
                E2J(J) = EXN - X*E1                             
                E3J(J) = (EXN - X*E2J(J))/2.
                E4J(J) = (EXN - X*E3J(J))/3.
                E5J(J) = (EXN - X*E4J(J))/4.                    
              ENDIF
            ELSE
            E1 = -LOG(X) + 
     1           ((((((((-0.310020E-5*X + 0.283447E-4)*X -      
     1           0.2314815E-3)*X + 0.1666667E-2)*X - 
     1           0.1041667E-1)*X + 0.5555556E-1)*X - 
     2           0.25)*X + 1.0)*X - 0.57721566)         
            E2J(J) = EXN - X*E1
            E3J(J) = (EXN - X*E2J(J))/2.
            E4J(J) = (EXN - X*E3J(J))/3.
            E5J(J) = (EXN - X*E4J(J))/4.                        
            ENDIF
          ENDIF
1     CONTINUE
C
C *** EXPONENTIAL INTEGRALS FOR ALL DELTA TAU
C
      DO 10 J=1,NN
        DO 11 I=2,NM1
         X = TA(J)-TA(I)
         IF (X.LT.0.) X=-X
         IF (X.EQ.0.) THEN
           E2(J,I) = 1.0                                                        
           E3(J,I) = 0.5                                                        
           E4(J,I) = 0.3333333333                                               
           E5(J,I) = 0.25                                                       
         ELSE          
          IF (X.GT.50.) THEN
             E2(J,I) = 0.0
             E3(J,I) = 0.0
             E4(J,I) = 0.0
             E5(J,I) = 0.0
          ELSE
            EXN = EXP(-X)                                                       
            IF (X .GE. 1.0) THEN
              IF (X .GT. 5.) THEN
                X2 = (X+9.)*(X+9.)                                              
                E9 = EXN/(X+9.)*((((54.*X*X-648.*X+729.)/X2+                    
     1               (81.-18.*X))/X2+9.)/X2+1.)                         
                E8 = (EXN - 8.*E9)/X                                            
                E7 = (EXN - 7.*E8)/X                                            
                E6 = (EXN - 6.*E7)/X                                            
                E5(J,I) = (EXN - 5.*E6)/X
                E4(J,I) = (EXN - 4.*E5(J,I))/X          
                E3(J,I) = (EXN - 3.*E4(J,I))/X  
                E2(J,I) = (EXN - 2.*E3(J,I))/X  
              ELSE
                E1 = EXN/X*
     1              (((X + 5.12669020)*X + 4.53079215)*X + 0.2372905)/
     1              (((X + 6.12652717)*X + 8.66601262)*X + 2.47663307)  
                E2(J,I) = EXN - X*E1                            
                E3(J,I) = (EXN - X*E2(J,I))/2.
                E4(J,I) = (EXN - X*E3(J,I))/3.
                E5(J,I) = (EXN - X*E4(J,I))/4.                  
              ENDIF
            ELSE
            E1 = -LOG(X) + 
     1           ((((((((-0.310020E-5*X + 0.283447E-4)*X -      
     1           0.2314815E-3)*X + 0.1666667E-2)*X - 
     1           0.1041667E-1)*X + 0.5555556E-1)*X - 
     2           0.25)*X + 1.0)*X - 0.57721566)         
            E2(J,I) = EXN - X*E1
            E3(J,I) = (EXN - X*E2(J,I))/2.
            E4(J,I) = (EXN - X*E3(J,I))/3.
            E5(J,I) = (EXN - X*E4(J,I))/4.                      
            ENDIF
          ENDIF
         ENDIF
11      CONTINUE
10    CONTINUE
      IF (Z .LT. 1.) GOTO 104                                           
      DO 109 J = 1,NN                                                   
      G(J,1) = 1.-0.5*E2J(J)
      H(J,1) = 2.*E3J(J)
      G(J,NN) = TA(J)+0.5*E3J(J)
      H(J,NN) = 4./3.-2.*E4J(J)                                 
      DO 103 I = 2,NM1                                                  
      TTA = TA(J)-TA(I)
      IF (TTA.LT.0.) THEN
         TTA = -TTA
         G(J,I) = ((2./3.-E4(J,I)-E4J(J)+TTA*TTA)/TA(I)-E3J(J))/TA(I)-
     1            0.5*E2J(J)
         H(J,I) = -4.*((E5(J,I)-E5J(J)+2./3.*TTA)/TA(I)-E4J(J))/TA(I)+
     1            2.*E3J(J)
      ELSE
         IF (TTA.EQ.0.) THEN
            G(J,I) = ((1./3.-E4J(J))/TA(J)-E3J(J))/TA(J)-0.5*E2J(J)
            H(J,I) = -4.*((0.25-E5J(J))/TA(J)-E4J(J))/TA(J)+2.*E3J(J)
         ELSE
            G(J,I) = ((E4(J,I)-E4J(J))/TA(I)-E3J(J))/TA(I)-0.5*E2J(J)
            H(J,I) = -4.*((E5(J,I)-E5J(J))/TA(I)-E4J(J))/TA(I)+
     1               2.*E3J(J)
         ENDIF
      ENDIF
103   CONTINUE
109   CONTINUE
      GOTO 111                                                          
C
C****  CALCULATE G AND H MATRICES FOR Z.LT.1
C
104   EMZ = 1.-Z                                                        
      EPZ = 1.+Z                                                        
      DO 110 J = 1,NN                                                   
      G(J,1) = 1.-0.5*E2J(J)
      H(J,1) = 2.*E3J(J)
      G(J,NN) = TA(J)+0.5*E3J(J)
      H(J,NN) = 4./3.-2.*E4J(J)
      DO 108 I = 2,NM1                                                  
      TTA=TA(J)-TA(I)
      IF (TTA.LT.0) THEN
         TTA = -TTA
         TA2 = TA(I)*TA(I)
         G(J,I) = (1.-TA(J)/TA(I))*(1.-Z*TA(J)/TA(I))+
     1            2.*Z/(3.*TA2)-
     1            0.5*E2J(J)+0.5/TA(I)*(EMZ*E3(J,I)-EPZ*E3J(J))-
     2            Z/TA2*(E4(J,I)+E4J(J))        
         H(J,I) = H(J,1)+2./TA(I)*(EMZ*E4(J,I)+EPZ*E4J(J)-2./3.*EPZ)-
     1            4.*Z/TA2*(E5(J,I)-E5J(J)-2./3.*TA(J))
         ELSE
            IF (TTA.EQ.0.) THEN
               TA2 = TA(J)*TA(J)
               G(J,I) = -0.5*E2J(J)+0.5/TA(J)*(EMZ/2.-EPZ*E3J(J))+
     1         Z/TA2*(1./3.- E4J(J))            
               H(J,I) = H(J,1)-2./TA(J)*(EMZ/3.-EPZ*E4J(J))-
     1                  4.*Z/TA2*(0.25-E5J(J))
            ELSE
               TA2 = TA(I)*TA(I)
               G(J,I) = -0.5*E2J(J)+0.5/TA(I)*(EMZ*E3(J,I)-EPZ*E3J(J))+
     1                  Z/TA2*(E4(J,I)-E4J(J))
               H(J,I) = H(J,1)-2./TA(I)*(EMZ*E4(J,I)-EPZ*E4J(J))-
     1                  4.*Z/TA2*(E5(J,I)-E5J(J))
            ENDIF
         ENDIF
108   CONTINUE                                                          
110   CONTINUE                                                          
111   RETURN                                                            
      END                                                               
      SUBROUTINE HYDRO(T,LM,CAP4,SIG,PP,PPPE,FR,GRAD,DENS,DIST,ISW)      

*+
*  NUMERISCHE INTEGRATION DER HYDROSTATISCHEN GLEICHUNG              
*-

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'COMMON_KIEL'

      DIMENSION PP(50),ZET(50),T(50),X(4,4),ALPHAX(7,50),CAP4(50),
     1   PPPE(50),SIG(50),TAUX(50),FR(15,50),GRAD(50),DIST(50),DINT(50),  
     2   DENS(50),ALQ(50)                                               

      COMMON /MODEL/  TEF,G,ALPHAX,WM                                      
      COMMON /PARAM/  LA,LL,NN,KK                                        
      COMMON /OPTDEP/ ZET,TAUX,A,Q,ALQ                                   

      DATA BC,C,SIGMA /1.38046E-16,2.99793E+10,5.6692E-05/              

*.

      IF (LM .EQ. 1) THEN                                             
        DO I = 1,NN                                                     
          GRAD(I) =  SIGMA*TEFF**4/C                                      
        ENDDO                                                        
        PP(1)  = (G*WM/0.6655E-24-GRAD(1))*TAUX(1)                      
      ELSE
        DO  K=1,5                                                      
          IF (G/CAP4(K)/GRAD(K) .LE. 1.5)  GRAD(K) = G/CAP4(K)/1.5          
        ENDDO                                                          
        PP(1) = (G/CAP4(1)-GRAD(1))*TAUX(1)                                
      ENDIF

C**** DAMIT IST DER ANFANGSWERT FESTGELEGT                              
C**** DIE ERSTEN VIER SCHRITTE NACH RUNGE-KUTTA                         
      K = 1                                                             
      H = 1.
      WREF = 4000.
                                                            
1     TAU1 = TAUX(K)                                                     
      PP1  = PP(K)                                                       
      T1   = T(K)                                                         
      GRD1 = GRAD(K)                                                    
      K1   = K+1
      J = K                                                          
      CALL PRESS(T1,PP1,PPE,X)
      CALL OPK_RBF (T1,PP1,PPE,WREF,CAPP,RBF,SIGM)                      
      CAP4(K) = CAPP                                              
      PPPE(K) = PPE                                                 
      SIG(K) = SIGM                                               
      DO I = 1,3                                                   
        FR(I,K) = X(1,I)
      ENDDO
      DO I = 1,4                                                   
        FR(I+3,K) = X(2,I)                                          
        FR(I+7,K) = X(3,I)                                             
        FR(I+11,K) = X(4,I)                                            
      ENDDO

      IF (K.LT.4) THEN                                                    
         F1 = TAU1*ALQ(K)*(G/CAPP-GRD1)                              
         ZETA2 = ZET(K)+H/2.                                         
         PP2 = PP(K)+H/2.*F1                                         
         TAU2 = A*Q**ZETA2                                           
         T2 = T(K)+(TAU2-TAUX(K))/(TAUX(K+1)-TAUX(K))*(T(K+1)-T(K))     
         GRD2 = GRAD(K)+(TAU2-TAUX(K))/(TAUX(K+1)-TAUX(K))*             
     :          (GRAD(K+1)-GRAD(K))                                         
         J= K + 1
         CALL PRESS(T2,PP2,PPE,X)                                     
         CALL OPK_RBF (T2,PP2,PPE,WREF,CAPP,RBF,SIGM)                     
         F2 = TAU2*ALQ(K)*(G/CAPP-GRD2)                              
         TAU3 = TAU2                                                 
         PP3 = PP(K)+H/2.*F2                                         
         T3 = T2                                                     
         GRD3 = GRD2                                                 
         CALL PRESS(T3,PP3,PPE,X)                                     
         CALL OPK_RBF (T3,PP3,PPE,WREF,CAPP,RBF,SIGM)                     
         F3 = TAU3*ALQ(K)*(G/CAPP-GRD3)                              
         TAU4 = TAU(K+1)                                             
         PP4 = PP(K)+H*F3                                            
         T4 = T(K+1)                                                 
         GRD4 = GRAD(K+1)                                            
         CALL PRESS(T4,PP4,PPE,X)                                     
         CALL OPK_RBF (T4,PP4,PPE,WREF,CAPP,RBF,SIGM)                      
         F4 = TAU4*ALQ(K)*(G/CAPP-GRD4)                              
         PP(K+1) = PP(K)+H/6.*(F1+2.*F2+2.*F3+F4)                    
         K = K+1                                                     
         GOTO 1                                                         
      ENDIF
                                                          
C**** DAMIT IST DIE BERECHNUNG DER ERSTEN VIER SCHRITTE NACH DEM        
C**** RUNGE-KUTTA-VERFAHREN BEENDET                                     
C**** FORTSETZUNG DER INTEGRATION NACH HAMMINGS PREDICTOR-CORRECTOR-    
C**** VERFAHREN                                                         
C**** DIE VIER ANFANGSWERTE DES DRUCKS PP(1), PP(2), PP(3), PP(4)       
C**** SIND JETZT BEKANNT                                                
C**** DIE VIER ABLEITUNGEN DES DRUCKS (BEZUEGLICH ZETA) F1, F2, F3, F4  
C**** WERDEN BENOETIGT.  BEACHTE DIE AENDERUNG DER BEDEUTUNG VON F2,    
C**** F3, F4 GEGENUEBER DEM RUNGE-KUTTA-VERFAHREN                       
      F2 = TAU(2)*ALQ(2)*(G/CAP4(2)-GRAD(2))                      
      F3 = TAU(3)*ALQ(3)*(G/CAP4(3)-GRAD(3))                      
      F4 = TAU(4)*ALQ(4)*(G/CAP4(4)-GRAD(4))                      

C**** PREDICTOR FUER DEN FUENFTEN PUNKT                                 
      PA = PP(1) +1.333333*H*(2.*F4-F3+2.*F2)                        
      J = 5
      CALL PRESS(T(5),PA,PPE,X)                                     
      CALL OPK_RBF (T(5),PA,PPE,WREF,CAPP,RBF,SIGM)                    
      FB = TAUX(5)*ALQ(5)*(G/CAPP-GRAD(5))                            
C**** CORRECTOR FUER DEN FUENFTEN PUNKT                                 
      PC = 0.125*(9.*PP(4)   -PP(2)   +3.*H*(FB+2.*F4-F3))           
      PP(5)    = PC+0.07438*(PA-PC)                                  
      CALL PRESS(T(5),PP(5),PPE,X)                                  
      CALL OPK_RBF (T(5),PP(5),PPE,WREF,CAPP,RBF,SIGM)
      F5 = TAUX(5)*ALQ(5)*(G/CAPP-GRAD(5))                            
      CAP4(5)   = CAPP                                              
      PPPE(5)    = PPE                                                 
      SIG(5)    = SIGM                                               
      DO I = 1,3                                                   
        FR(I,5)    = X(1,I)                                            
      ENDDO
      DO I = 1,4                                                  
        FR(I+3,5)  = X(2,I)                                          
        FR(I+7,5)  = X(3,I)                                            
        FR(I+11,5) = X(4,I)                                            
      ENDDO

      MM = 6                                                         
12    F2 = F3                                                     
      F3 = F4                                                     
      F4 = F5                                                     

C**** PREDICTOR FUER DEN PUNKT MM                                       
      PAA = PP(MM-4)+1.333333*H*(2.*F4-F3+2.*F2)                  

C**** MODIFIER FUER DEN PUNKT MM                                        
      PBB =  PAA-0.9256198*(PA-PC)                                
      IF (PBB .LT. 0.0)  PBB = 1.E-6                              
      J = MM
      CALL PRESS(T(MM),PBB,PPE,X)                                  
      CALL OPK_RBF (T(MM),PBB,PPE,WREF,CAPP,RBF,SIGM)          
      FBB = TAUX(MM)*ALQ(MM)*(G/CAPP-GRAD(MM))                     

C**** CORRECTOR FUER DEN PUNKT MM                                       
      PCC = 0.125*(9.*PP(MM-1)-PP(MM-3)+3.*H*(FBB+2.*F4-F3))      
      PP(MM) = PCC+0.07438*(PAA-PCC)                              
      J = MM
      CALL PRESS(T(MM),PP(MM),PPE,X)                                       
      CALL OPK_RBF (T(MM),PP(MM),PPE,WREF,CAPP,RBF,SIGM)
      F5       = TAUX(MM)*ALQ(MM)*(G/CAPP-GRAD(MM))                      
      CAP4(MM) = CAPP                                             
      PPPE(MM) = PPE                                                
      SIG(MM)  = SIGM                                              
      DO I = 1,3                                                  
        FR(I,MM) = X(1,I)                                           
      ENDDO
      DO I = 1,4                                                  
        FR(I+3,MM)  = X(2,I)                                         
        FR(I+7,MM)  = X(3,I)                                            
        FR(I+11,MM) = X(4,I)                                           
      ENDDO

      IF (MM.LT.NN) THEN
         PA = PAA                                                    
         PC = PCC                                                    
         MM = MM+1                                                   
         GOTO 12                                                        
      ENDIF
                                                          
C**** AUF DIESE ART WERDEN ALLE NN PUNKTE BERECHNET                     
C**** GEOMETRISCHE TIEFE UND MASSENDICHTE                               
      DO K = 1,NN                                                 
         DENS(K) = WM*(PP(K)-PPPE(K))/(BC*T(K))                       
         DINT(K) = TAUX(K)*ALQ(K)/(CAP4(K)*DENS(K))                   
      ENDDO
      DIST(1)  = TAUX(1) *0.5/(CAP4(1) *DENS(1) )                     
      CALL INGRAT( 1,NN,H,DIST,DINT)                                 

*     write(6,*) '   T      taux   dist   P      Pe     cap4   sig  '
*     write(6,1200) (T(j),taux(j),dist(j),pp(j),pppe(j),
*    :               cap4(j),sig(j),j=1,nn)
1200  format(0Pf8.1,1p6e12.3)

      END     
      SUBROUTINE INALPHA(GLG,ALPC,ALPN)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ALPMET/ ALPMET(7)
      COMMON/MODEL/TEF,G,ALPHA(7,50),WM
      DIMENSION ELEM(11)
      DIMENSION EPSSOL(9),ATW(9),BETSOL(9)
      DATA ELEM/ 1HH,2HHE,1HC,1HN,1HO,2HMG,2HAL,2HSI,1HS,2HCA,2HFE/
      DATA EPSSOL/ 8.67, 7.99, 8.92, 7.53, 6.43, 7.50, 7.20, 6.36, 7.46/
      DATA ATW/ 12.011, 14.007, 16.000, 24.313, 26.982, 28.09, 32.066,
     A          40.08, 55.849/

C-----------------------------------------------------------------------
C**** FESTLEGUNG DER ELEMENT HAEUFIGKEITEN BEI VORGEGEBENEN H,HE,C,N-
C**** HAEUFIGKEITEN
C**** NUR H,HE,C,N,O,MG,AL,SI,S,CA,FE WERDEN BERUECKSICHTIGT.
C**** ALPC,ALPN UND ALPMET ENTHALTEN ZUNAECHST DIE HAEUFIGKEITEN RELATIV
C**** ZUR SOLAREN MISCHUNG (ALS DELTA EPSILON)
C**** DELTA EPSILON = -9.9 BEWIRKT ALPHA =0.0
C**** SOLARE HAEUFIGKEITEN NACH HOLWEGER, 1979, LIEGE CALL.
C**** SUMSOL = ALPHA(H)*1.008 + ALPHA(HE)*4.002 SOLAR
C**** BETSOL = SOLARER MASSENBRUCHTEIL
C-----------------------------------------------------------------------

      SUMALP = 0.0
      DO 10 J=1,7
         SUMALP = SUMALP + ALPHA(J,50)
10    CONTINUE
      ALPDIF = SUMALP - 1.
      IF (ABS(ALPDIF).GT.0.01) THEN
         WRITE(6,*) (ALPHA(J,50),J=1,7)
         STOP 
      ENDIF

30    CONTINUE
      SUMSOL = 1.284
      DO 1 I=1,9
      BETSOL(I) = 10**(EPSSOL(I)-12.)*ATW(I)
      SUMSOL = SUMSOL + BETSOL(I)
1     CONTINUE
      DO 2 I=1,9
2     BETSOL(I) = BETSOL(I)/SUMSOL
C**** NORMIERUNG DER AKTUELLEN ELEMENTHAEUFIGKEITEN AUF MASSENBRUCHTEIL
      SUMMA = 1.008*ALPHA(1,50) + 4.002*ALPHA(2,50)
      SUMMA = SUMMA + ATW(1)*ALPHA(6,50) + ATW(2)*ALPHA(7,50)
      IF (ALPHA(6,50).EQ.0.) ALPHA(6,50)=10**ALPC*BETSOL(1)/ATW(1)*SUMMA
      IF (ALPC.EQ.-9.9) ALPHA(6,50) = 0.0
      IF (ALPHA(7,50).EQ.0.) ALPHA(7,50)=10**ALPN*BETSOL(2)/ATW(2)*SUMMA
      IF (ALPN.EQ.-9.9) ALPHA(7,50) = 0.0
      DO 3 I=1,7
3     ALPMET(I) = 10**ALPMET(I)*BETSOL(I+2)/ATW(I+2)*SUMMA
      VMICRO=5.
      WRITE(6,1315) TEF,GLG,VMICRO
      WRITE(6,1110) (ELEM(I),ALPHA(I,50),I=1,2),
     A              (ELEM(I),ALPHA(I+3,50),I=3,4),
     A              (ELEM(I+4),ALPMET(I),I=1,7)
1315  FORMAT(3X,'STELLARE PARAMETER:',16X,'TEF = ',F6.0,14X,'LOG G = ',
     1F5.2,14X,'VMIC = ',F4.1,/,3X,19('-'),16X,12('-'),14X,13('-'),14X,
     211('-')/,3X,'ELEMENTHAEUFIGKEITEN')
1110  FORMAT(3X,A10,E12.3)
      RETURN
      END
      SUBROUTINE INDAT(WL,WL5,L3,WLSWP)

*+
*
*  Program:
*     INDAT
*
*  Purpose:
*     Reads in the wavelength grid to STERNE from file WAVE
*
*-

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PARAM/ LA,LL,NN,KK

      DIMENSION WL(3035),WL5(L3)

      DATA NK / 7 /

*.

      DO 100 K=1,3035
100   WL(K)=0.0
      DO 200 K=1,L3
200   WL5(K) = 0.0

*  Read Wavelength grid.
      OPEN(NK,FILE='WAVE',STATUS='OLD',FORM='FORMATTED')
      READ(NK,*) LA,LL
      READ(NK,*) (WL(I),I=LA,LL)
      CLOSE(NK)  
      L3 = LL
      DO I=LA,LL
         WL5(I)=WL(I)
      ENDDO

      END
      SUBROUTINE INGRAT(NA,NN,H,QUA,TEG)                                
C**** INTEGRATIONSROUTINE, TAU-INTEGRATION                              
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION QUA(NN),TEG(NN)                                         
      NM2 = NN-2                                                        
      NM3 = NN-3                                                        
      NA1 = NA+1                                                        
      DO 1 J = NA1,NM3                                                  
    1    QUA(J) = QUA(J-1)+H/48.*(17.*TEG(J-1)+42.*TEG(J)-16.*TEG(J+1)  
     1         +6.*TEG(J+2)-TEG(J+3))                                   
         DO 2 J = NM2,NN                                                
    2    QUA(J) = QUA(J-2)+H/3.*(TEG(J-2)+4.*TEG(J-1)+TEG(J))           
      RETURN                                                            
      END                                                               
      SUBROUTINE INTP(K,L,Y,X,N,KS)                                     
C***  INTERPOLATION                                                     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N),Y(N)                                               
      KP1 = K+1                                                         
      LM1 = L-1                                                         
      LP1 = L+1                                                         
      IF (K .LE. 1) GOTO 20                                             
      DO 10 J = KP1,LM1                                                 
   10    Y(J) = Y(L)-(Y(L)-Y(K))*(X(L)-X(J))/(X(L)-X(K))                
         KS = 1                                                         
      RETURN                                                            
   20 IF (Y(LP1) .LT. 0.) RETURN                                        
      DO 30 J = 1,LM1                                                   
   30    Y(J) = Y(L)-(Y(LP1)-Y(L))*(X(L)-X(J))/(X(LP1)-X(L))            
         KS = 1                                                         
      RETURN                                                            
      END                                                               

      SUBROUTINE KONTOP(WLI,WL5,L3,T,X,I,ISW,CAC)                       

C**** BERECHNUNG DER KONTINUUMSOPAZITAETEN AN DEN STUETZSTELLEN WL5     
C**** DAZWISCHEN LINEARE INTERPOLATION                                  

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION WL5(L3),CACL(50),CACR(50),X(4,4),CAC(50),T(50)          
      DIMENSION SIGL(50),SIGR(50)                                       

      INCLUDE 'COMMON_KIEL'

      COMMON /ABSKO/  CAP4(50),SIG(50)                                   
      COMMON /PARAM/  LA,LL,NN,KK                                        
      COMMON /PARKON/ LA3
      COMMON /DRUCK/  PP(50),PPE(50),FR(15,50)                            


*  The shortest and longest wavelengths must be continuum wavelengths
      IF (I.EQ.LA) THEN
        L5 = LA3
        L6 = LA3 + 1
        DO J=1,NN
          CALL OPK_RBF (T(J),PP(J),PPE(J),WL5(L5),OPAC,CACL(J),SIGL(J))
        ENDDO
      ELSE
        IF (ABS(WLI-WL5(L6)).GT.0.01) GOTO 100
        IF (I.EQ.LL) GOTO 200
        DO J=1,NN
          CACL(J) = CACR(J)
          SIGL(J) = SIGR(J)
        ENDDO
        L5 = L5 + 1
        L6 = L6 + 1
      ENDIF

      DO J=1,NN
        CALL OPK_RBF (T(J),PP(J),PPE(J),WL5(L6),OPAC,CACR(J),SIGR(J))
      ENDDO

100   DWL5 = (WLI-WL5(L5)) / (WL5(L6)-WL5(L5))
      DO J=1,NN
         CAC(J) = CACL(J) + (CACR(J)-CACL(J)) * DWL5
         SIG(J) = SIGL(J) + (SIGR(J)-SIGL(J)) * DWL5
      ENDDO

*     write (6,*) T
*     write (6,*) CAC
*     write (6,*) SIG

      RETURN                                                            

200   DO J=1,NN                                                      
         CAC(J) = CACR(J)                                               
         SIG(J) = SIGR(J)                                               
      ENDDO                                                          

      END                                                               

*DK LINOP
      SUBROUTINE LINOP(T,EN,FREQ,N,NSTEPS,STEPWT)

C***   EINLESEN DER ODFS, MODIFIZIERTE ROUTINE VON KURUCZ
C***   ODFS UEBER KANAL 10

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PARAM/ LA,LL,NN,KK
      COMMON /ODF/ CAPODF(50)
      COMMON /ITEMP/ ITEMP

      DIMENSION T(50),EN(50)
      DIMENSION STEPS(9,29,10)
      DIMENSION WT(10)
      DIMENSION CO1(50),CO2(50),CO3(50),CO4(50),INEJ(50),ITJ(50)
      DIMENSION TABT(29),TABNE(9)

      DATA TABT/3.48,3.52,3.56,3.60,3.64,3.68,3.72,3.76,3.80,3.84,3.88,
     : 3.92,3.96,4.0,4.05,4.10,4.15,4.20,4.25,4.30,4.35,4.40,4.45,4.5,
     : 4.6,4.7,4.8,4.9,5.0/
      DATA TABNE/17.,16.,15.,14.,13.,12.,11.,10.,9./
      DATA ITEMP1/0/
      DATA ICOUNT/50/
      DATA TENLOG/2.30258509299405E0/

      IF(ITEMP.EQ.ITEMP1)GO TO 20
      ITEMP1=ITEMP
      DO 15 J=1,NN
      TL=LOG10(T(J))
      DO 10 IT=2,29
      IF(TL.LT.TABT(IT))GO TO 11
   10 CONTINUE
      IT=29
   11 XNELOG=LOG10(EN(J))
      DO 12 INE=2,9
      IF(XNELOG.GT.TABNE(INE))GO TO 13
   12 CONTINUE
      INE=9
   13 INEJ(J)=INE
      ITJ(J)=IT
      X=(TL-TABT(IT-1))/(TABT(IT)-TABT(IT-1))
      Y=(XNELOG-TABNE(INE-1))/(TABNE(INE)-TABNE(INE-1))
      CO1(J)=(1.-X)*(1.-Y)*TENLOG
      CO2(J)=(1.-X)*Y*TENLOG
      CO3(J)=X*(1.-Y)*TENLOG
      CO4(J)=X*Y*TENLOG
   15 CONTINUE
      WT(1)=1./60.
      WT(2)=2./60.
      WT(3)=3./60.
      WT(4)=6./60.
      WT(5)=6./60.
      WT(6)=6./60.
      WT(7)=6./60.
      WT(8)=10./60.
      WT(9)=10./60.
      WT(10)=10./60.
C     OPEN (10,FILE='ODF',STATUS='OLD',FORM='UNFORMATTED')
      REWIND 10
      WLEND=0.
C**** ACHTUNG WELLENLAENGE IN NANOMETER
   20 WAVE=2.997925E17/FREQ
   21 IF(WAVE.LT.WLEND)GO TO 22
      READ(10,END=999) WLBEG,WLEND,STEPS
      GO TO 21
   22 NSTEPS=10
      STEPWT=WT(N)
      DO 25 J=1,NN
      IT=ITJ(J)
      INE=INEJ(J)
   25 CAPODF(J)=EXP(CO1(J)*STEPS(INE-1,IT-1,N)+CO2(J)*STEPS(INE,IT-1,N)+
     1CO3(J)*STEPS(INE-1,IT,N)+CO4(J)*STEPS(INE,IT,N))
      RETURN
999     STOP 'END DATA'
      END
      SUBROUTINE OPK_RBF (T,PG,PE,WL,OPA,RBF,RFF)

*+
*  Program:
*     OPK_RBF

*  Purpose:
*     Interface STERNE (interim code) with OPK library

*        Inputs:
*        T, PG, PE, WL
*
*        Outputs:
*        OPA    - total opacity
*        RBF    - absorption coefficient
*        RFF    - scattering coefficient

*-

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION TT, PGJ, PEJ, WLS, OPAC, CAP, SIG
      DOUBLE PRECISION T,  PG,  PE,  WL,  OPA,  RBF, RFF

*.

      TT  = T
      PGJ = PG
      PEJ = PE
      WLS = WL

      CALL OPKC (TT,PGJ,PEJ,WLS,OPAC,CAP,SIG)

      OPA = OPAC
      RBF = CAP
      RFF = SIG

      END
      SUBROUTINE OUT1(T,DELT,DENS,DIST,DEL,K,LM,LK,TNEU,WL,FNUE,LMAX,
     A IFL)

C**** OUTPUT TAUABHAENGIGE GROESSEN                                     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION TAUX(50),T(50),DELT(50),PP(50),PPE(50),    
     1 FR(15,50),CAP4(50),SIG(50),CAPP(50),CAPJ(50),CAPF(50),DENS(50),  
     2   DIST(50),ZET(50),ALPHAX(7,50),GRAD(50),FLX(50),TNS(50),DEL(50), 
     3  ALQ(50),B(50),DELB1(50),DELB2(50),TNEU(50),FNUE(LMAX),WL(LMAX)  
      DIMENSION TAU5(50)
      DOUBLE PRECISION M1                                                           

      INCLUDE 'COMMON_KIEL'

      COMMON /MODEL/   TEF,G,ALPHAX,WM                                     
      COMMON /PARAM/   LA,LL,NN,KK                                        
      COMMON /OPTDEP/  ZET,TAUX,P,Q,ALQ                                   
      COMMON /DRUCK/   PP,PPE,FR                                          
      COMMON /ABSKO/   CAP4,SIG                                           
      COMMON /INTGRA/  FLX,GRAD,TNS,CAPP,CAPJ,CAPF,DELB1,DELB2,B         
      COMMON /KAP5/    CAPP5(50)
      COMMON /FINT/    F1239,WLSWP,FSWP,F3121
      COMMON /ITCOR/   CORREC(60),NIT
      COMMON /SCHICH/  IEL,TAU0

*.

      LA1 = LA
      LL1 = LL
      SIGMA = 5.6692E-05                                                
      GLG = LOG10(G)                                                   
C***  H- UND HE- HAEUFIGKEIT AM INNEREN RAND. BEI  GESCHICHTETEN ATMOS-
C***  PHAEREN HAUFIGKEIT AM AEUSSEREN RAND BEACHTEN
      AHYD = 100.*ALPHAX(1,50)                                         
      AHEL = 100.*ALPHAX(2,50)                                                
      ACAR = 100.*ALPHAX(6,50)                                           
      ANIT = 100.*ALPHAX(7,50)                                           

      PI = 4.*ATAN(1.)                                                  
      FF = SIGMA*TEF**4/PI                                              
      DO 1777 J=LA,LL
1777  FNUE(J) = 1.0E18*2.99793*FNUE(J)/WL(J)/WL(J)
      DO  J = 1,NN                                                   
        PG(J) = LOG10(PP(J))                                             
        PE(J) = LOG10(PPE(J))
        TAU5(J) = CAPP5(J)/CAP4(J)*TAUX(J)
      ENDDO
      NLP = 5
      IF (LK.EQ.0) THEN
        NLP = 1
        WRITE(K,304)                                                      
        WRITE(K,700)                                                      
      ENDIF
      IF (LK.NE.0) THEN
*       Only print out every 3rd iteration.
        IF (LM/3*3.NE.LM) RETURN
        WRITE(K,301) LM                                                   
        NLP = 5
      ELSEIF (IEL.EQ.0) THEN
        WRITE(K,321) LM
      ELSE                                            
        WRITE(K,322) LM,ALPHAX(2,50),ALPHAX(2,1),TAU0
      ENDIF
      WRITE(K,305)                                                      
      WRITE(K,306) TEF,GLG,FF,AHYD,AHEL,ACAR,ANIT                       
      WRITE(K,307)                                                      
      WRITE(K,308) (I,TAUX(I),TAU5(I),T(I),DELT(I),PG(I),PE(I),CAP4(I),  
     1CAPP5(I),SIG(I),DENS(I),DIST(I),GRAD(I),FLX(I),DEL(I),I=1,NN,NLP)
      IF (LK.NE.0) RETURN

      WRITE(6,309) WL(LA),F1239,WLSWP,FSWP,WL(LL),F3121
      WRITE(K,304)
      WRITE(K,1000)
      WRITE(K,1100) (I,CORREC(I),I=1,NIT)
      WRITE(K,304)                                                      

*  Print out the ionisation equilibrium.
      WRITE(K,100)                                                      
      WRITE(K,103)                                                      
      WRITE(K,104) (J,TAUX(J),FH(1,J),FHE(1,J),FHE(2,J),
     :             (FC(I,J),I=1,4),(FN(I,J),I=1,4),J=1,NN)

* Print out the final flux distribution.
      IF (LK .NE. 0) GOTO 350                                           
      LZ = LA-1
345   WRITE(K,500)                                                      
      WRITE(K,510)                                                      
      DO 800 J =1,50
      IF (FNUE(LZ+J+200).EQ.0.0) GOTO 810
      WRITE(K,520) WL(LZ+J),FNUE(LZ+J),WL(LZ+J+50),FNUE(LZ+J+50),
     1          WL(LZ+J+100),FNUE(LZ+J+100),WL(LZ+J+150),FNUE(LZ+J+150),
     2          WL(LZ+J+200),FNUE(LZ+J+200)
      GOTO 800
810   IF (FNUE(LZ+J+150).EQ.0.0) GOTO 820
      WRITE(K,520) WL(LZ+J),FNUE(LZ+J),WL(LZ+J+50),FNUE(LZ+J+50),
     1          WL(LZ+J+100),FNUE(LZ+J+100),WL(LZ+J+150),FNUE(LZ+J+150)
      GOTO 800
820   IF (FNUE(LZ+J+100).EQ.0.0) GOTO 830
      WRITE(K,520) WL(LZ+J),FNUE(LZ+J),WL(LZ+J+50),FNUE(LZ+J+50),
     1          WL(LZ+J+100),FNUE(LZ+J+100)
      GOTO 800
830   IF (FNUE(LZ+J+50).EQ.0.0) GOTO 840
      WRITE(K,520) WL(LZ+J),FNUE(LZ+J),WL(LZ+J+50),FNUE(LZ+J+50)
      GOTO 800
840   IF (FNUE(LZ+J).EQ.0.0) GOTO 410
      WRITE(K,520) WL(LZ+J),FNUE(LZ+J)
800   CONTINUE

*  Write out the flux distribution to the output file.
      LZ = LZ+250                                                       
      IF (LL.GT.LZ) GOTO 345                                            
410   IF (IFL.NE.1) GOTO 610
      REWIND 8
      WRITE(8) TEF,G,(ALPHAX(I,50),I=1,7),WM
      DO I=LA,LL
         IF (WL(I).LT.2800.) THEN
            LA1 = I
         ELSEIF (WL(I).GT.8000.) THEN
            GOTO 560
         ENDIF
      ENDDO
560   LL1 = I
      LA1LL1 = LL1-LA1+1
      WRITE(8) LA1LL1
      DO I=LA1,LL1
         WRITE(8) WL(I),FNUE(I)
      ENDDO

*  Write out the model to the binary file
610   CONTINUE
      REWIND 2                                                          
      WRITE(2) TEF,GLG,FF,AHYD,AHEL,ACAR,ANIT                           
      WRITE(2) (TAUX(I),T(I),DELT(I),PG(I),PE(I),CAP4(I),SIG(I),         
     1   DENS(I),DIST(I),GRAD(I),FLX(I),DEL(I),I = 1,NN)                
      WRITE(2) (FR(1,I),FR(3,I),FR(4,I),FR(5,I),FR(7,I),FR(8,I),        
     1   FR(9,I),FR(10,I),FR(11,I),FR(12,I),FR(13,I),FR(14,I),FR(15,I), 
     2   I=1,NN)                                                        
      REWIND 2                                                          
      RETURN                                                            

  350 WRITE(K,304)                                                      
      WRITE(K,250)                                                      
  360 WRITE(K,210)                                                      
      WRITE(K,220) (I,CAPP(I),CAPJ(I),CAPF(I),B(I),DELB1(I),DELB2(I),   
     1      TNEU(I), I = 1,NN)                                          
      RETURN                                                            

304   FORMAT (1H1)                                                      
305   FORMAT(23X,'TEFF',7X,'LOG G',7X,'FLUX',10X,'H%',9X,'HE%', 9X,     
     :   'C%',10X,'N%',/)                                               
306   FORMAT(20X,F8.1,5X,F5.2,5X,1PE9.3,4(4X,1PE8.2))                   
307   FORMAT(//,2X,2HNR,5X,3HTAU,6X,4HTAU5,6X,1HT,7X,4HDELT,3X,5HLOGPG,
     :2X,5HLOGPE,3X,5HKAPPA,3X,5HKAPP5,3X,5HSIGMA,5X,3HRHO,7X,4HDIST,6X,
     :   8HDPR/DTAU,4X,4HFLUX,8X,5HDF/F=,/)                             
308   FORMAT(1X,I3,1PE10.3,1PE10.3,0PF8.1,F9.1,2F7.3,3F8.4,1PE11.3,     
     :   3E11.3,0PF10.3)                                                
309   FORMAT(/,3X,'FLUSSINTEGRALE: ',F8.1,'A BIS 1239A F=',E12.4,'  BIS
     :   ',F8.1,'A F=',E10.4,'  UND 3121.8 BIS ',F8.1,'A F=',E10.4) 
100   FORMAT(//'   IONISATIONSBRUCHTEILE '//)                           
103   FORMAT(2X,'NR',5X,'TAU',8X,'HI',8X,'HEI',9X,'HEII',7X,'CI',8X,    
     :   'CII',8X,'CIII',7X,'CIV',9X,'NI',9X,'NII',9X,'NIII',8X,'NIV',/)
104   FORMAT(1X,I3,1PE11.3,11E11.3)                                     
301   FORMAT(//,3X,I3,24H.TE TEMPERATURSCHICHTUNG,//)                   
321   FORMAT ('   KONVERGIERTES MODELL ',I3,' ITERATIONEN HOMOGEN',//)
322   FORMAT ('   KONVERGIERTES MODELL ',I3,' ITERATIONEN GESCHICHTET:',
     :   ' N(HE) = ',F6.3,' (INNEN), N(HE) = ',F6.3,' (AUSSEN), TAU0 = ',
     :    E10.4,//)                
250   FORMAT(//'   TEMPERATURKORREKTION '//)                            
210   FORMAT(//,7X,2HNR,8X,4HKAPP,10X,4HKAPJ,10X,4HKAPF,10X,1HB,11X,    
     :   5HDELB1,9X,5HDELB2,10X,4HTNEU,/)                               
220   FORMAT(5X,I4,1PE14.2,5E14.2,0PF14.1)                              
500   FORMAT(1H1,'   FLAMBDA AN DER OBERFLAECHE')                       
510   FORMAT(//,6X,6HLAMBDA,5X,4HFLAM,8X,6HLAMBDA,5X,4HFLAM,8X,         
     :  6HLAMBDA,5X,4HFLAM,8X,6HLAMBDA,5X,4HFLAM,8X,6HLAMBDA,5X,4HFLAM, 
     :  /)                                                              
520   FORMAT(3X,0PF10.3,1PE11.3,2X,0PF10.3,1PE11.3,2X,0PF10.3,          
     : 1PE11.3,2X,0PF10.3,1PE11.3,2X,0PF10.3,1PE11.3)                   
600   FORMAT(//,'  D(HEI 3422) = ',F7.4,5X,'D(HI 3646) = ',F7.4)        
700   FORMAT(20X,'  L T E - S T E R N A T M O S P H A E R E N P R O G R 
     :A M M  I N S T .  F .  A S T R O P H Y S I K ')                   
1000    FORMAT(10X,'KONVERGENZINDIKATOR SQRT( SUM(DELTA T QUDRAT))',/,
     : 10X,'ITER.   CORREC')
1100    FORMAT(10X,I4,3X,E12.4)

      END                                                               

      SUBROUTINE OUTLINE
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/CODE/ CODE1(435),CODE2(435),MC1,MC2
      COMMON /LDAT/ EP(435),GF(435),WL0(435),GAMRAD(435),GAMEL(435),
     1             DWL0(435),N,DWL1(435),GAMVW(435)
      DO 10 K=1,N
10    GF(K) = LOG10(GF(K))
      WRITE(6,1000)
      JMIN = MIN0(250,N)
      WRITE(6,1010) (WL0(J),CODE1(J),CODE2(J),EP(J),GF(J),J=1,JMIN)
      IF (N.LE.250) GOTO 100
      WRITE(6,1000)
      WRITE(6,1010) (WL0(J),CODE1(J),CODE2(J),EP(J),GF(J),J=251,N)
100   RETURN
1000  FORMAT(1H1,//3X,'LINIENLISTE:',//,5('   LAMBDA EL. IO  EP   GF '),
     1/)
1010  FORMAT(5(F9.1,F4.0,F3.0,F5.1,F5.1))
      END

      SUBROUTINE OUTSTAT
C**** AUSGABE DER STATISTIK UEBER GENAUIGKEIT DER QUELLFUNKTION
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/QSTATIS/ KIMAX,KST,FRST(5),ERST(5)
      DATA C /2.99793E18/
      DO 100 I=1,5
      IF (FRST(I).EQ.0.0) GOTO 100
      FRST(I) = C/FRST(I)
100   CONTINUE
      WRITE(6,1000) KIMAX,KST
      WRITE(6,1010)
      WRITE(6,1020) (FRST(I),ERST(I),I=1,5)
      RETURN
1000  FORMAT(//,3X,'QUELLFUNKTION NACH ',I2,' ITERATIONEN AN ',I4,
     1 'WELLENLAENGENPUNKTEN NICHT KONVERGIERT')
1010  FORMAT(3X,' 5 PUNKTE MIT GROESSTEM QUELLFUNKTIONSFEHLER:')
1020  FORMAT(3X,'WL= ',F8.1,4X,'ERR= ',E9.3)
      END
      SUBROUTINE PRESS(TT,P,PEL,X)                                    

*+

*  Program:
*     PRESS

*  Purpose:
*     To evaluate electron pressure by iteration of ioniastion equations for
*     H, He, C and N

*  Input arguments:
*     TT      Temperature (K)
*     P       Gas pressure (dyn / cm^2)

*  Output arguments:
*     PEL     Electron pressure (dyn / cm^2)
*     X       Ionisation fractions for H, He, C, N                                   
*-

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'COMMON_KIEL'

      DIMENSION F(4,3),X(4,4),ALPHAX(7,50),XM(7,4)                                     
      DOUBLE PRECISION DTT, DPE
      DOUBLE PRECISION FX(6), ZU(42)
      DOUBLE PRECISION SUM

      DOUBLE PRECISION U

      INTEGER ZZ(7)

      COMMON /MODEL/   TEF,G,ALPHAX,WM                                      
      COMMON /ALPMET/  ALPMET(7)                                         

      DATA SAHA0 / 0.666698474 /, BK / 8.6167E-05 /                     

      DATA  ZU / 9., 4., 9., 6., 1., 2. ,
     :           1., 2., 1., 6., 9., 4. ,
     :           6., 1., 2., 1., 6., 9. ,      
     :           9., 6., 1., 2., 1., 6. ,
     :           9., 4., 9., 6., 1., 2. ,
     :           1., 2., 1., 6., 9., 4. ,     
     :          25.,30.,25., 6., 7., 6. /
      DATA  ZZ / 8, 12, 13, 14, 16, 20, 26 /  

*.

      SAHA = SAHA0*TT**2.5                                              
      BKT = BK*TT                                                       
      FH1 = SAHA*EXP(-13.595/(BKT))                                     
      FHM = SAHA*EXP(-0.75/(BKT))                                       
      FHE1 = SAHA*EXP(-24.581/(BKT))                                    
      FHE2 = SAHA*EXP(-54.403/(BKT))                                    
      FHEM = SAHA*EXP(+0.075/BKT)                                       
      DO K = 1,4                                                      
        DO I = 1,4                                                      
          X(I,K) = 0.                                                       
        ENDDO
      ENDDO
      K = 0                                                             
      IF (TEF .GE. 12000.)  THEN                                   
        F(1,1) = FHE1*2.                                            
        X0 = SQRT(F(1,1)/(P+F(1,1)))                                
      ELSE                                                         
        FC1 = SAHA*EXP(-11.256/BKT)
        F(3,1) = FC1*6./9.                                          
        X0 = SQRT(F(3,1)/(P+F(3,1)))                                
      ENDIF
      PEL0 = P*X0/(1.+X0)                                          

C**** START ITERATION FOR ELECTRON PRESSURE
5     CONTINUE
*     convert T,Pe to double precision units for PF calculation
      DTT = TT
      DPE = PEL0

C**** WASSERSTOFF                                                       
      IF (ALPHAX(1,J) .GT. 0.) THEN                                        
        ZH1 = U (1,1,DTT,DPE)                                            
        F(1,1) = FH1/ZH1                                            
        F(1,2) = FHM*ZH1                                            
        FHJ(2) = F(1,1)*F(1,2)/(PEL0**2+PEL0*F(1,2)+F(1,1)*F(1,2))    
        FHJ(3) = (1.-FHJ(2))*PEL0/(PEL0+F(1,2))                       
        FHJ(1) = 1 - FHJ(2) - FHJ(3)
      ENDIF

C**** HELIUM                                                            
      IF (ALPHAX(2,J) .GT. 0.) THEN                 
        ZHE1 = U (2,1,DTT,DPE)                                          
        ZHE2 = U (2,2,DTT,DPE)                                          
        F(2,1) = FHE1*ZHE2/ZHE1                                     
        F(2,2) = FHE2/ZHE2                                          
        F(2,3) = FHEM*ZHE1/6.                                       
        FHEJ(2) = PEL0*F(2,1)*F(2,3)/(PEL0**3+PEL0**2*F(2,3)+PEL0*       
     :            F(2,1)*F(2,3)+F(2,1)*F(2,2)*F(2,3))                       
        FHEJ(3) = FHEJ(2)*F(2,2)/PEL0                                  
        FHEJ(4) = FHEJ(2)*PEL0**2/(F(2,1)*F(2,3))                      
        FHEJ(1) = 1 - FHEJ(2) - FHEJ(3) - FHEJ(4)
      ENDIF

C**** KOHLENSTOFF (neglecting C- )
      IF (ALPHAX(6,J) .GT. 0.) THEN
        DO I = 1,6
          ZCJ(I) = U (6,I,DTT,DPE)                                         
        ENDDO
        CALL ION ( 6, DTT, DPE, ZCJ, FCJ )
      ENDIF

C**** STICKSTOFF                                                        
      IF (ALPHAX(7,J) .GT. 0.) THEN
        DO I = 1,6
          ZNJ(I) = U (7,I,DTT,DPE)                                         
        ENDDO
        CALL ION ( 7, DTT, DPE, ZNJ, FNJ )
      ENDIF

C**** TOTAL                                  
      SUM = ALPHAX(1,J)*(FHJ(2)-FHJ(3)) +
     :      ALPHAX(2,J)*(FHEJ(2)+2*FHEJ(3)-FHEJ(4)) +
     :      ALPHAX(6,J)*(FCJ(2)+2*FCJ(3)+3*FCJ(4)+4*FCJ(5)+5*FCJ(6)) +
     :      ALPHAX(7,J)*(FNJ(2)+2*FNJ(3)+3*FNJ(4)+4*FNJ(5)+5*FNJ(6))

C**** OTHER ELEMENTS
      PM = 0.0                                                          
      DO  K = 1,7                                                     
        CALL ION ( ZZ(K), DTT, DPE, ZU(6*K-5), FX )
        PM = PM + ALPMET(K) * 
     :            (FX(2) + 2*FX(3) + 3*FX(4) +4*FX(5) + 5*FX(6))
        DO  I=1,4
           XM(K,I) = FX(I)
        ENDDO
      ENDDO

      SUM = SUM + PM                                                 

C**** CHECK FOR CONVERGENCE
      PEL = P/(1.+1./SUM)                                             
      IF (ABS(1.-PEL0/PEL) .GE. 0.0002)  THEN
        K = K+1                                                     
        PEL0 = (PEL0+PEL)/2.                                            
        IF (K.GT.30)  WRITE (6,18)  PEL0,PEL                        
        IF (K.LE.30)  GOTO 5                                               
      ENDIF
                                                         
C**** LOAD COMMON BLOCKS FOR OPACITY
      ZHJ(1) = ZH1
      ZH(1,J) = ZH1
      DO I = 1,3
         FH(I,J) = FHJ(I)
      ENDDO
      ZHEJ(1) = ZHE1
      ZHEJ(2) = ZHE2
      ZHE(1,J) = ZHE1
      ZHE(2,J) = ZHE2
      DO I = 1,4
         FHE(I,J) = FHEJ(I)
      ENDDO

      DO I = 1,6
         ZC(I,J) = ZCJ(I)
         FC(I,J) = FCJ(I)
         ZN(I,J) = ZNJ(I)
         FN(I,J) = FNJ(I)
      ENDDO

      DO I = 1,4
         FOJ(I)   = XM(1,I)
         FMGJ(I)  = XM(2,I)
         FALJ(I)  = XM(3,I)
         FSIJ(I)  = XM(4,I)
         FSJ(I)   = XM(5,I)
         FCAJ(I)  = XM(6,I)
         FO(I,J)  = XM(1,I)
         FMG(I,J) = XM(2,I)
         FAL(I,J) = XM(3,I)
         FSI(I,J) = XM(4,I)
         FS(I,J)  = XM(5,I)
         FCA(I,J) = XM(6,I)
      ENDDO

   18 FORMAT(2X,2E14.5,2X,'ELEKTRONENDRUCK KONVERGIERT NICHT')          
      END                                                     
      SUBROUTINE STAND(GLG,ZT,T,FF,NN,KK,LM)                            

C***  BERECHNUNG DER STANDARDWERTE                                      

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ALPHA(7,50),ATW(7),TAU(50),T(50),ZET(50),ALQ(50)                
      COMMON /MODEL/ TEF,G,ALPHA,WM                                     
      COMMON /OPTDEP/ ZET,TAU,P1,Q1,ALQ                                 
      DATA HM,SIGMA /1.6732E-24,5.6692E-05/                             
      DATA ATW/ 1.00801,4.0028,6.939,9.013,10.812,12.01161,14.0073/     


*  Mean molecular weight of stellar material in grams.
      WM = 0.                                                           
      DO 10 I = 1,KK                                                    
   10   WM = WM+HM*ALPHA(I,50)*ATW(I)/ATW(1)                           

*  Surface gravity (real value).
      G = 10.0**GLG                                                     

*  Optical depth scale.
      P1 = 10**(-6.2)
      Q1 = 10**(0.5)
      P2 = 10**(-5.)
      Q2 = 10**(0.2)
      P3 = 10.**(-3.)                                                   
      Q3 = 10.**0.1                                                     
      NM1 = NN-1                                                        
      ZET(1) = ZT                                                       
      DO 11 I = 1,NM1                                                   
   11   ZET(I+1) = ZET(I)+1.                                           
      DO 14 I=1,4
        TAU(I) = P1*Q1**ZET(I)                                         
   14   ALQ(I) = LOG(Q1)                                              
      DO 15 I=5,20
        TAU(I) = P2*Q2**ZET(I)                                            
   15   ALQ(I) = LOG(Q2)                                                 
      DO 16 I = 21,NN                                                   
        TAU(I) = P3*Q3**ZET(I)                                         
   16   ALQ(I) = LOG(Q3)                                              

*  Initial temperature values (grey atmosphere).
      IF (LM .EQ. 1) THEN
      DO 30 I = 1,NN                                                    
   30   T(I) = SQRT(SQRT(0.75*TEF**4*(TAU(I)+0.577351)))               
      ENDIF

      FF = SIGMA*TEF**4/(4.*ATAN(1.))                                                  
      END          
                                                     
        SUBROUTINE STRATI(IEL,TAU0)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /MODEL/ TEF,G,ALPHA(7,50),WM
        COMMON /OPTDEP/ ZET(50),TAU(50),A,Q,ALQ(50)
C****  ELEMENTSCHICHTUNG FUER WASSERSTOFF UND HELIUM
C****  FALLS IEL NE 0
C****  VORBESETZEN ALLER FELDELEMENTE AUSSER FUER H UND HE 
C****  AM AUESSEREN RAND (1. TIEFENPUNKT)
        DO 712 J=1,7
          IF (J.GT.2) ALPHA(J,1) = ALPHA(J,50)
          DO 712 I=2,50
            ALPHA(J,I) = ALPHA(J,50)
712     CONTINUE
        IF (IEL.NE.0) THEN
          K=1/IEL*IEL + 1
          DO 713 I=1,49
            X = TAU(I)/TAU0
            IF (X.LT.1.) THEN
              ALPHA(IEL,I) = ALPHA(IEL,1)*EXP(LOG(ALPHA(IEL,50)
     A                                        /ALPHA(IEL,1)) *X*X)
              ALPHA(K,I) = 1.-ALPHA(IEL,I)
            ENDIF
713       CONTINUE
          WRITE(6,*) 'ELEMENTSCHICHTUNG FUER H, HE VORGEGEBEN:'
          WRITE(6,*) 'AUSSEN: H = ',ALPHA(1,1),' HE = ',ALPHA(2,1)
          WRITE(6,*) 'TAU0 = ',TAU0
          WRITE(6,1000) 
          WRITE(6,1100) (TAU(K),ALPHA(1,K),ALPHA(2,K),K=1,50)
        ELSE
          DO 714 J=1,7
            ALPHA(J,1) = ALPHA(J,50)
714       CONTINUE
        ENDIF
        RETURN
1000    FORMAT(/,1X,'   TAU           H       HE ',/)
1100    FORMAT(1X,E12.4,2F8.3)
        END
      SUBROUTINE TAUNUE(CAP)                                            

*+
*  Program:
*     TAUNUE

*  Purpose:
*     Calculation of tau_nu

*  Description:
*     Establishes the optical depth scale at a frequency nu, 
*     given the continuous opacity at this frequency.

*-

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /STN_TAUN/ TA(50),Z                                          
      COMMON /OPTDEP/ ZET(50),TAU(50),P,Q,ALQ(50)                       
      COMMON /PARAM/ LA,LL,NN,KK                                        
      COMMON /ABSKO/ CAP4(50),SIG(50)                                   

      DIMENSION TEG(50),CAP(50)                                         
      DATA ONE / 1.0D0 /

*.

      DO  J=1,NN                                                      
        TEG(J) = TAU(J)*ALQ(J)*(CAP(J)+SIG(J))/CAP4(J)                  
      ENDDO

      TA(1) = TEG(1)*0.5/ALQ(1)                                         
      IF (SIG(1).GT.CAP(1)) TA(1) = 2.*TA(1)                            
      CALL INGRAT(1,NN,ONE,TA,TEG)                                       

*     write (6,*) ' tau    teg    ta     cap    sig    cap4   alq '
*     write (6,1200) (tau(j),teg(j),ta(j),cap(j),sig(j),cap4(j),
*    :                alq(j),j=1,nn)
1200  format (1p7e12.2)

      DO  J=2,NN                                                      
        K = J                                                           
        IF (TA(J).LE.TA(J-1)) GOTO 30
      ENDDO
      RETURN                                                            

30    DO  JJ =K,NN                                                    
        TA(JJ) = TA(JJ-1)+0.5*(TEG(JJ-1)+TEG(JJ))                       
      ENDDO
      RETURN                                                            

      END 
                                                              
      SUBROUTINE TEMKOR(FLX,TNS,CAPP,CAPJ,CAPF,CAP4,T,TNEU,             
     1      DELB1,DELB2,B,ISW)                                          

C**** TEMPERATURKORREKTION NACH LUCY                                    
C**** MIT KONVERGENZBESCHLEUNIGUNG NACH AUER

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION FLX(50),TNS(50),CAPP(50),CAPJ(50),CAPF(50),CAP4(50),    
     1   T(50),TNEU(50),DEL(50),DELB1(50),DELB2(50),B(50),ALPHA(7,50),  
     2   TEG(50),ZET(50),TAU(50),ALQ(50),BINN(50)                       
      DIMENSION T4(0:3,50),DELT(50)

      COMMON /MODEL/ TEF,G,ALPHA,WM                                     
      COMMON /PARAM/ LA,LL,NN,KK                                        
      COMMON /OPTDEP/ ZET,TAU,P,Q,ALQ                                   
      COMMON /KONVK/ DENS(50),V(50),FC(50),DIST(50)                     
      COMMON /ITCOR/ CORREC(60),NIT

        SAVE T4
        DATA T4 /200*0.0/
        DATA NIT /0/
*.

         PI = 4.*ATAN(1.)                                               
         SIGMA = 5.6692E-05                                             
         NM1 = NN-1                                                     
         F  = SIGMA*TEF**4/PI                                           
         DO 1 J = 1,NN                                                  
      BINN(J) = CAPJ(J)/CAPP(J)*B(J)                                    
         CAPF(J) = CAPF(J)/FLX(J)                                       
1     DELB1(J) = BINN(J)-B(J)                                           
         DEL(1) = 0.                                                    
         FH =FLX(1)/TNS(1)                                              
         FK = 1./3.                                                     
         DO 2 J = 1,NN                                                  
         FF = F                                                   
    2    TEG(J) = 0.25*CAPF(J)/CAP4(J)*(FLX(J)-FF)                      
         DO 23 J = 1,NM1                                                
   23    DEL(J+1) = DEL(J)+0.5*(TEG(J+1)+TEG(J))*(TAU(J+1)-TAU(J))      
         DO 4 J = 1,NN                                                  
         FKJ = FK+0.0768*EXP(-SQRT(TAU(J)))                             
         FF = F                                                   
4        DELB2(J) = -BINN(J)/TNS(J)*((FLX(1)-FF)/FH+DEL(J)/FKJ)         
         DO 7 J = 1,NN                                                  
         BINN(J) = DELB1(J)+DELB2(J)                                    
C******* KORREKTURBESCHRAENKUNGEN AUSGESCHALTET
         IF (BINN(J)/B(J)-0.1) 6,5,5                                    
   5     BINN(J) = 0.8*DELB1(J)+DELB2(J)                                
    6    IF (ABS(BINN(J))/B(J) .LT. 0.2) GOTO 7                         
         IF (ISW .LE. 1) GOTO 7                                         
         SM = ABS(BINN(J))/BINN(J)                                      
         BINN(J) = 0.2*B(J)*SM                                          
C***************************************************************
    7    TNEU(J) = T(J)*(1.+BINN(J)/B(J)*0.25)                          
C****   SAVE LAST 3 TEMPERATURE STRATIFICATIONS PLUS THE NEW ONE
C****   FOR USE IN ACCEL
        NIT = NIT+1
        DO 100 I=1,50
           T4(3,I) = T4(2,I)
           T4(2,I) = T4(1,I)
           T4(1,I) = T(I)
           T4(0,I) = TNEU(I)
100     CONTINUE
        SUM = 0.
        DO 200 I=1,50
           DELT(I) = (T4(0,I)-T4(1,I))/T4(1,I)
           SUM = SUM+DELT(I)*DELT(I)
200     CONTINUE        
        CORREC(NIT) = SQRT(SUM)
        IF ((NIT/4)*4.NE.NIT.OR.NIT.LT.5) RETURN
C****   APPLY LARRY AUERS SECOND ORDER ACCELERATION METHOD
C****   TO GET IMPROVED TEMPERATURE STRATIFICATION
        CALL ACCEL(T4,50,TNEU)
      END                                                               
